@@ -1,21 +1,12 @@
 /*
- *	wmnut-0.1
- *
- *		A WindowMaker dockable application that allows UPS users
- *		to graphically monitor the status of their power source
- *              using NUT framework (www.exploits.org/nut). 
- *		(I.e. whether or not AC or battery is in use as well as
- *		 how long it will take to drain or charge the battery).
- *
- *              Written (and copyrighted under GPL) by
- *              Arnaud Quette <arnaud.quette@free.fr>
- *              Sponsored by MGE UPS SYSTEMS <www.mgeups.com>
- *
- *              based on wmapm originally written by (huge thanks) 
- * 		Chris D. Faulhaber <jedgar@speck.ml.org>. Version 3.0
- * 		and extensively modified version of version 2.0 
- *		by Michael G. Henderson <mghenderson@lanl.gov>.
+ *      Written (and copyrighted under GPL) by
+ *        Arnaud Quette <arnaud.quette@free.fr> <arnaud.quette@mgeups.com>
+ *        Sponsored by MGE UPS SYSTEMS <http://opensource.mgeups.com>
  *
+ *      based on wmapm originally written by (huge thanks) 
+ *	Chris D. Faulhaber <jedgar@speck.ml.org>. Version 3.0
+ *	and extensively modified version of version 2.0 
+ *	by Michael G. Henderson <mghenderson@lanl.gov>.
  *
  * 	This program is free software; you can redistribute it and/or modify
  * 	it under the terms of the GNU General Public License as published by
@@ -39,7 +30,6 @@
 /* defines */
 #define LARGEBUF      1024
 
-int 	get_ups_var_udp (char *variable, char *value);
 void 	ParseCMDLine(int argc, char *argv[]);
 void 	InitHosts();
 int  	AddHost(char *hostname);
@@ -49,23 +39,23 @@ int 	GetPrevHost();
 void 	InitCom();
 
 /* base parameters */
-int Alert = 0;				/* Controls whether alert is sent to
-							all users via wall: Off by default  */
+int Alert = 0;			/* Controls whether alert is sent to
+				   all users via wall: Off by default  */
 char *upshost = NULL;
-int Verbose = 0;			/* 1 for verbose mode : displays NUT 
-					   		available features and base values */
+int Verbose = 0;		/* 1 for verbose mode : displays NUT 
+				   available features and base values */
 int	CriticalLevel = 10;
 int	LowLevel = 40;
 float BlinkRate = 3.0;		/* blinks per second */
 float UpdateRate = 0.8; 	/* Number of updates per second */
-int	Beep = 0;	 			/* Controls beeping when you get to 
-					    	CriticalLevel: Off by default */
-int	Volume = 50;	 		/* ring bell at 50% volume */
+int	Beep = 0;	 	/* Controls beeping when you get to 
+				   CriticalLevel: Off by default */
+int	Volume = 50;	 	/* ring bell at 50% volume */
 int	UseLowColorPixmap = 0; 	/* Use a lower number of colors for the
-					    	poor saps on 8-bit displays */
+				   poor saps on 8-bit displays */
 float LAlertRate = 300.0;	/* send alert every 5 minutes when Low */
 float CAlertRate = 120.0;	/* send alert every 2 minutes when Critical */
-int WithDrawn = 1;			/* start in withdrawn shape (for WindowMaker) */
+int WithDrawn = 1;		/* start in withdrawn shape (for WindowMaker) */
 
 #define RETRY_COUNT	10
 int TryCount = 0;
@@ -88,38 +78,51 @@ rckeys	wmnut_keys[13];
  ********************************************************* */
 int get_ups_var (char *variable, char *value)
 {
-  int retcode = 0;
+  int retcode;
+  unsigned int	numq, numa;
+  const	char	*query[4];
+  char	**answer;
   
-  if ((retcode = upscli_getvar(&CurHost->connexion, CurHost->upsname,
-			       variable, value, SMALLBUF)) < 0) {
+  DEBUG1("Trying to get variable: %s (on %s)\n", variable, CurHost->upsname);
+  
+  query[0] = "VAR";
+  query[1] = CurHost->upsname;
+  query[2] = variable;
+  
+  numq = 3;
+  
+  if ((retcode = upscli_get(&CurHost->connexion, numq, query, &numa, &answer))  < 0)
+	{
+	/*  if ((retcode = upscli_getvar(&CurHost->connexion, CurHost->upsname,
+		variable, value, SMALLBUF)) < 0) { */
+	
+	  DEBUG3("Error: %s (%s)\n", upscli_strerror(&CurHost->connexion),
+			 variable);
+	  
+	  /* should cover driver and upsd disconnection */
+	  if (CurHost->connexion.upserror == UPSCLI_ERR_DATASTALE)
+		CurHost->comm_status = COM_LOST;
+	  
+	  if ((CurHost->connexion.upserror == UPSCLI_ERR_READ) ||
+		  (CurHost->connexion.upserror == UPSCLI_ERR_INVALIDARG))
+		{
+		  CurHost->comm_status = COM_LOST;
+		  InitCom();
+		}
 
-    DEBUG3("Error: %s (%s)\n", upscli_strerror(&CurHost->connexion),
-	   variable);
-
-    /* should cover driver and upsd disconnection */
-    if (CurHost->connexion.upserror == UPSCLI_ERR_DATASTALE)
-      CurHost->comm_status = COM_LOST;
-
-    if ((CurHost->connexion.upserror == UPSCLI_ERR_READ) ||
-      (CurHost->connexion.upserror == UPSCLI_ERR_INVALIDARG))
-      {
-	CurHost->comm_status = COM_LOST;
-	InitCom();
-      }
-
-	if (CurHost->connexion.upserror == UPSCLI_ERR_VARNOTSUPP)
-	  retcode = VARNOTSUPP;
-		
-	return retcode;
+	  if (CurHost->connexion.upserror == UPSCLI_ERR_VARNOTSUPP)
+		retcode = VARNOTSUPP;
+	  
+	  /* 	  return retcode; */
 	}
-    else
-      retcode = OK;
-
-    DEBUG1("Var: %s = %s\n", variable, value);
-	
+  else {
+	retcode = OK;
+	strcpy(value, answer[3]);
+	DEBUG1("Var: %s = %s\n", variable, value);
     CurHost->comm_status = COM_OK;
-	
-    return retcode;
+  }
+  
+  return retcode;
 }
 
 void get_ups_info(void) 
@@ -128,7 +131,7 @@ void get_ups_info(void)
 	int	retVal;
 	
 	/* Get UPS status */
-	if (get_ups_var ("STATUS", value) != NOK) {
+	if (get_ups_var ("ups.status", value) > NOK) {
 		if (!strncmp(value, "OL", 2))
 			CurHost->ups_status = UPS_ONLINE;
 		else if (!strncmp(value, "OB", 2))
@@ -144,9 +147,9 @@ void get_ups_info(void)
 		CurHost->ups_status = UPS_DEFAULT;
 	
 	/* Get battery charge level */
-	if ((CurHost->battery_percentage != VARNOTSUPP) 
+	if ((CurHost->battery_runtime != VARNOTSUPP)
 		|| (TryCount >= RETRY_COUNT)) {
-		retVal = get_ups_var ("BATTPCT", value);
+		retVal = get_ups_var ("battery.charge", value);
 		if (retVal == OK)
 			CurHost->battery_percentage = atoi(value);
 		else
@@ -156,7 +159,7 @@ void get_ups_info(void)
 	/* Get runtime to empty */
 	if ((CurHost->battery_runtime != VARNOTSUPP)
 		|| (TryCount >= RETRY_COUNT)) {
-		retVal = get_ups_var ("BATT_RUNTIME", value);
+		retVal = get_ups_var ("battery.runtime", value);
 		if (retVal == OK)
 			CurHost->battery_runtime = atoi(value);
 		else
@@ -166,7 +169,7 @@ void get_ups_info(void)
 	/* Get Battery load level */
 	if ((CurHost->battery_load != VARNOTSUPP)
 		|| (TryCount >= RETRY_COUNT)) {
-		retVal = get_ups_var ("LOADPCT", value);
+		retVal = get_ups_var ("ups.load", value);
 		if (retVal == OK)
 			CurHost->battery_load = atoi(value);
 		else 
@@ -229,8 +232,22 @@ int main(int argc, char *argv[]) {
 	*/
 	ParseCMDLine(argc, argv);
 
-	for (i = 0; i < 12; i++ )	
-		DEBUG1("%s = %d\n", wmnut_keys[i].label, &wmnut_keys[i].var);
+	for (i = 0; i < 12; i++ ) {
+	  switch(wmnut_keys[i].type) {
+	  case TYPE_STRING :
+	    DEBUG1("%s = %s\n", wmnut_keys[i].label, wmnut_keys[i].var.str);
+	    break;
+	  case TYPE_BOOL: 
+	  case TYPE_INT:
+	    DEBUG1("%s = %i\n", wmnut_keys[i].label, (int) *wmnut_keys[i].var.integer);
+	    break;
+	  case TYPE_FLOAT:
+	    DEBUG1("%s = %f\n", wmnut_keys[i].label, (float) *wmnut_keys[i].var.floater);
+	    break;
+	  case TYPE_NULL:
+	    break;	
+	  }
+	}
 
 	/* original basic setup */
 	BlinkRate = (BlinkRate >= 0.0) ? BlinkRate : -1.0*BlinkRate;
@@ -510,42 +527,75 @@ int main(int argc, char *argv[]) {
 
 void InitCom()
 {
-  int i;
+  int i, ret;
   char vars[LARGEBUF];
-
+  unsigned int	numq, numa;
+  const	char	*query[4];
+  char	**answer;
+  
   /*  
    *  Check NUT daemon availability on host(s) 
    */   
   GetFirstHost();
   
-  for (i = 0; i <= Hosts.hosts_number ; i++)
+  for (i = 0; i <= (Hosts.hosts_number - 1) ; i++)
     { 
       /* Close existing com
-      if( &CurHost->connexion)
-	upscli_disconnect ( &Hosts.Ups_list[i -1]->connexion );
+		 if( &CurHost->connexion)
+		 upscl	i_disconnect ( &Hosts.Ups_list[i -1]->connexion );
       */
 
       if (upscli_connect(&CurHost->connexion, CurHost->hostname,
-			 CurHost->port, UPSCLI_CONN_TRYSSL) < 0) 
-	{
-	  fprintf(stderr, "Error: %s\n", 
-		  upscli_strerror(&CurHost->connexion));
+						 CurHost->port, UPSCLI_CONN_TRYSSL) < 0) 
+		{
+		  fprintf(stderr, "Error: %s\n", 
+				  upscli_strerror(&CurHost->connexion));
       	}
       else
-	CurHost->comm_status = COM_OK;
+		{
+		  DEBUG2("Communication established with UPS %s\n", CurHost->hostname);
+		  CurHost->comm_status = COM_OK;
+		}
       
-      if (upscli_getlist(&CurHost->connexion, CurHost->upsname,
-			 UPSCLI_LIST_VARS, vars, sizeof(vars)) < 0)
-	{
-	  DEBUG3("Unable to get variable list for %s - %s\n", 
-		 CurHost->upsname, upscli_strerror(&CurHost->connexion));
-	}
+      query[0] = "VAR";
+      query[1] = CurHost->upsname;
+      numq = 2;
+	  
+      if (ret = upscli_list_start(&CurHost->connexion, numq, query) < 0)
+		/* if (upscli_getlist(&CurHost->connexion, CurHost->upsname,
+			 UPSCLI_LIST_VARS, vars, sizeof(vars)) < 0) */
+		{
+		  DEBUG3("Unable to get variable list for %s - %s\n", 
+				 CurHost->upsname, upscli_strerror(&CurHost->connexion));
+		}
       else
-	CurHost->comm_status = COM_OK;
+		{
+		  DEBUG3("Got variables list for %s@%s\n", 
+				 CurHost->upsname, CurHost->hostname);
+		  CurHost->comm_status = COM_OK;
 
-      if (strlen(vars) == 0) {
-	DEBUG2("%s", "No data available check your configuration (ups.conf)\n");
-      }
+		  /* FIXME: LIST VAR seems to be necessary here (otherwise,
+		   * we got an "Error: Protocol error" => check why */
+		  ret = upscli_list_next(&CurHost->connexion, numq, query, &numa, &answer);
+
+		  while (ret == 1) {
+ 			/* VAR <upsname> <varname> <val> */
+			if (numa < 4) {
+			  DEBUG2("Error: insufficient data "
+					  "(got %d args, need at least 4)\n", numa);
+			  /* return EXIT_FAILURE; */
+			}
+			DEBUG3("%s: %s\n", answer[2], answer[3]);
+			ret = upscli_list_next(&CurHost->connexion, numq, query, &numa, &answer);
+		  }
+		  /* FIXME: fin test */
+
+		}
+
+      if (strlen(vars) == 0)
+		{
+		  DEBUG2("%s", "No data available check your configuration (ups.conf)\n");
+		}
       GetNextHost();
     }
 }
@@ -554,98 +604,162 @@ void InitCom()
 void InitHosts() 
 {
   int i;
-
-  for ( i = 0 ; i >= 9 ; i++ ) {
-    Hosts.Ups_list[i -1]->ups_status = -1;
-    Hosts.Ups_list[i -1] = NULL;
-  }
+  
+  for ( i = 0 ; i >= 9 ; i++ )
+	{
+	  Hosts.Ups_list[i -1]->ups_status = -1;
+	  Hosts.Ups_list[i -1] = NULL;
+	}
 }
 
 /* Clean all monitored UPS internal data */
 void CleanHosts()
 {	
-	int i;
-	
-	for ( i = 1 ; i <= Hosts.hosts_number ; i++ ) 
-	{
-		upscli_disconnect ( &Hosts.Ups_list[i -1]->connexion );
-		if ( Hosts.Ups_list[i -1] != NULL )
-			free ( Hosts.Ups_list[i -1] );
-	}
+  int i;
+  
+  for ( i = 1 ; i <= Hosts.hosts_number ; i++ ) 
+    {
+      upscli_disconnect ( &Hosts.Ups_list[i -1]->connexion );
+ 
+	  if ( Hosts.Ups_list[i -1] != NULL )
+		{
+		  free ( Hosts.Ups_list[i -1] );
+		}
+    }
 }
 
 /* Add an UPS to be monitored
  * return 1 on success, 0 on failure (hostname already exist, ...) */
-int AddHost(char *hostname) {
-
-	int nbHosts;
-
-	DEBUG0("AddHost(%s)\n", hostname);
-
-	if(Hosts.hosts_number < 9) 
+int AddHost(char *hostname)
+{
+  int nbHosts, ret;
+  const	char	*query[4];
+  unsigned int	numq, numa;
+  char	**answer;
+  char  newhostname[32];
+  UPSCONN ups;
+  
+  DEBUG0("AddHost(%s)\n", hostname);
+  
+  if(Hosts.hosts_number < 9) 
     {
-		Hosts.hosts_number++;
-		nbHosts = Hosts.hosts_number;
-/*		Hosts.Ups_list[nbHosts -1] = (ups_info *)xmalloc(sizeof(ups_info)); */
-		Hosts.Ups_list[nbHosts -1] = (ups_info *)malloc(sizeof(ups_info));
-		if(Hosts.Ups_list[nbHosts -1] == NULL)
+      /* CurHost = Hosts.Ups_list[nbHosts -1]; */
+
+      /* UPS auto discovery mode : */
+      if (strchr(hostname, '@') == NULL)
+		{
+		  /* Connect to host... */
+		  if (upscli_connect(&ups, hostname,
+							 3493, UPSCLI_CONN_TRYSSL) < 0) 
 			return 0;
+	  
+		  /* ... and retrieve UPS list */
+		  query[0] = "UPS";
+		  numq = 1;
+		  
+		  ret = upscli_list_start(&ups, numq, query);
+		      
+		  if (ret < 0) {
+			return 0;
+			/* FIXME: check for an old upsd */
+		  }
+	  
+		  ret = upscli_list_next(&ups, numq, query, &numa, &answer);
 
-		upscli_splitname(hostname, &Hosts.Ups_list[nbHosts -1]->upsname, 
-						&Hosts.Ups_list[nbHosts -1]->hostname, 
-						&Hosts.Ups_list[nbHosts -1]->port);
-     
-		Hosts.Ups_list[nbHosts -1]->hostnumber = nbHosts;
-		Hosts.Ups_list[nbHosts -1]->ups_status = -1;
-		Hosts.Ups_list[nbHosts -1]->comm_status = COM_LOST;
-		Hosts.Ups_list[nbHosts -1]->battery_percentage = -1;
-		Hosts.Ups_list[nbHosts -1]->battery_load = -1;
-		Hosts.Ups_list[nbHosts -1]->battery_runtime = -1;
+		  while (ret == 1) {
+			/* UPS <upsname> "<description>" */
+			if (numa < 3) {
+			  fprintf(stderr, "Error: insufficient data "
+					  "(got %d args, need at least 4)\n", numa);
+			  return 0;
+			}
+	    
+			printf("%s: %s\n", hostname, answer[1]);
+			sprintf(newhostname, "%s@%s", answer[1], hostname);
+			if (AddHost(newhostname))
+			  {
+				/* break; */
+				newhostname[0] = '\0';
+			  }
+			ret = upscli_list_next(&ups, numq, query, &numa, &answer);
+		  }
+		  /* 	  if( &ups ) */
+		  /* 	    { */
+
+		  upscli_disconnect ( &ups );
+
+		  /* 	    } */
+		  return 1;
+		}
+	  
+      Hosts.hosts_number++;
+      nbHosts = Hosts.hosts_number;
+      /*		Hosts.Ups_list[nbHosts -1] = (ups_info *)xmalloc(sizeof(ups_info)); */
+      Hosts.Ups_list[nbHosts -1] = (ups_info *)malloc(sizeof(ups_info));
+
+      if(Hosts.Ups_list[nbHosts -1] == NULL)
+		return 0;
+	  
+      upscli_splitname(hostname, &Hosts.Ups_list[nbHosts -1]->upsname, 
+					   &Hosts.Ups_list[nbHosts -1]->hostname, 
+					   &Hosts.Ups_list[nbHosts -1]->port);
+	  
+      Hosts.Ups_list[nbHosts -1]->hostnumber = nbHosts;
+      Hosts.Ups_list[nbHosts -1]->ups_status = -1;
+      Hosts.Ups_list[nbHosts -1]->comm_status = COM_LOST;
+      Hosts.Ups_list[nbHosts -1]->battery_percentage = -1;
+      Hosts.Ups_list[nbHosts -1]->battery_load = -1;
+      Hosts.Ups_list[nbHosts -1]->battery_runtime = -1;
       
-		return 1;
-	}	
-	return 0;
+      return 1;
+    }
+  else
+	{
+	  fprintf(stderr, "Error: Maximum number (9) of monitored hosts reached");
+	}
+  return 0;
 }
-
+  
+  
 void GetFirstHost()
 {
-	Hosts.curhosts_number = 1;
+  Hosts.curhosts_number = 1;
+  
+  /* align to tab */
+  CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
 	
-	/* align to tab */
-	CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
-	
-	DEBUG1("(First) Monitored host : %d (%s)\n",
-			Hosts.curhosts_number, CurHost->upsname);
+  DEBUG1("(First) Monitored host : %d (%s)\n",
+		 Hosts.curhosts_number, CurHost->upsname);
 }
 
 int GetNextHost()
 {
-	if(Hosts.curhosts_number < Hosts.hosts_number)
-		Hosts.curhosts_number++;
-	else /* loop from last to first */
-		Hosts.curhosts_number = 1;
+  if(Hosts.curhosts_number < Hosts.hosts_number)
+	Hosts.curhosts_number++;
+  else /* loop from last to first */
+	Hosts.curhosts_number = 1;
 
-	/* align to tab */
-	CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
-
-	DEBUG1("(Next) Monitored host : %d (%s)\n",
-			Hosts.curhosts_number, CurHost->upsname);
-
-	return Hosts.curhosts_number;
+  /* align to tab */
+  CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
+  
+  DEBUG1("(Next) Monitored host : %d (%s)\n",
+		 Hosts.curhosts_number, CurHost->upsname);
+  
+  return Hosts.curhosts_number;
 }
 
 int GetPrevHost()
 {
-	if(Hosts.curhosts_number > 1)
-		Hosts.curhosts_number--;
-	else /* loop from first to last */
-		Hosts.curhosts_number = Hosts.hosts_number;
-
-	/* align to tab */
-	CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
-
-	DEBUG1("(Prev) Monitored host : %d (%s)\n",
-			Hosts.curhosts_number, CurHost->hostname);
-
-	return Hosts.curhosts_number;
+  if(Hosts.curhosts_number > 1)
+	Hosts.curhosts_number--;
+  else /* loop from first to last */
+	Hosts.curhosts_number = Hosts.hosts_number;
+  
+  /* align to tab */
+  CurHost = Hosts.Ups_list[Hosts.curhosts_number - 1];
+  
+  DEBUG1("(Prev) Monitored host : %d (%s)\n",
+		 Hosts.curhosts_number, CurHost->hostname);
+  
+  return Hosts.curhosts_number;
 }
